from fastapi import APIRouter, Request, Depends, Security
from fastapi.encoders import jsonable_encoder
from fastapi_pagination import Params
from api.db.database import get_db
from api.v1.routes.user import get_users
from api.v1.services.user import user_service
from api.v1.services.task import task_service
from api.v1.routes.task import available_tasks, fetch_task_submissions, get_task, fetch_mytasks, get_task_by_id, task_statistics
from api.v1.services.coin_token import token_services

from sqlalchemy.orm import Session

from fastapi.responses import HTMLResponse, RedirectResponse
from api.utils.config import templates_env

adminview = APIRouter(prefix="/admin", tags=["Admin VIews and UI"])

Title = "Admin Dashboard"


def dashboard_items(request: Request, db: Session = Depends(get_db)):
    user_id = user_service.fetch_user_refresh_token(request, db=db)

    if user_id == None:
        return None
    else:
        user = user_service.get_user_by_id(db=db, id=user_id)
        if not user.is_super_admin:
            return None
        else:
            wallet = user_service.fetch_user_walletbalance(user_id, db=db)
            return {"user": user, "balance": wallet}

def admin_check(db, user_id):
    user = user_service.get_user_by_id(db=db, id=user_id)
    if not user.is_super_admin:
        return True
    return None


@adminview.get("/login", response_class=HTMLResponse)
def login(request: Request):
    """Endpoint for log in form  a user"""
    if adminview.get:
        template = templates_env.get_template("main.html")
    return template.render({"request": request, 
         "page": "admin/login", 
         "title": "Admin Login", 
         "data": "",
         "tag" : "Welcome to Scrap3",
         "templates_env": templates_env,
         "isLogin": False})


@adminview.get("/dashboard", response_class=HTMLResponse)
def dashboard(request: Request, db: Session = Depends(get_db)):
    user_id = user_service.fetch_user_refresh_token(request, db=db)
    #print(user_id)
    if user_id is None:
        return RedirectResponse("/admin/login")
    user = user_service.get_user_by_id(db=db, id=user_id)
    #print(user)
    if not user.is_super_admin:
        return RedirectResponse("/admin/login")
    
    users_count = user_service.users_count(db)
    task_stats = task_statistics(db)
    wallet = user_service.fetch_user_walletbalance(user_id, db=db)
    template = templates_env.get_template("/admin/main.html")
    # print(wallet, user)
    return template.render(
        {
            "request": request,
            "title": Title,
            "tag" : "My Account",
            "page": "admin/dashboard",
            "user": user,
            "data": {"balance": wallet, "user": user, "users_count": users_count, 
                     "task_stats": task_stats,
                     },
            "templates_env": templates_env,
            "isLogin": True
        }
    )


@adminview.get("/alltasks", response_class=HTMLResponse)
async def view_jobs(request: Request, db: Session = Depends(get_db),  params: Params=Depends()):
    items = dashboard_items(request, db=db)
    if items is None:
        return RedirectResponse("/admin/login")
    
    tasks = await get_task(db=db, params=params,
        current_user = Security(user_service.get_current_user)
    )
    tasks = jsonable_encoder(tasks)
    template = templates_env.get_template("admin/main.html")

    return template.render(
        {
         "request": request, 
         "page": "admin/tasks", 
         "title": Title, 
         "tag" : "All Tasks",
         "data" :{"user": items["user"], "tasks":tasks.get('data'),},  
         "user": items["user"],
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )

@adminview.get("/users", response_class=HTMLResponse)
async def all_users(request: Request, db: Session = Depends(get_db),  params: Params=Depends()):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/admin/login")
    
    template = templates_env.get_template("admin/main.html")

    users = await get_users(db=db,page=1,per_page=10,is_active=None, is_deleted=None,is_verified=None, is_super_admin=None)
    """ p = users['data']
    print(p.email) """
    return template.render(
        {
         "request": request, 
         "page": "admin/allusers", 
         "title": Title, 
         "tag" : "All Users",
         "data" :{"user": users['data'],},  
         "user": items["user"],
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )


#Create Task View Admin
@adminview.get("/create-task", response_class=HTMLResponse)
async def create_task(request: Request, db: Session = Depends(get_db)):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/admin/login")
    """Password Reset View"""
    user = items["user"]
    token_balances = await token_services.user_token_balances(
        db, user_id=user.id)
    tokens = token_balances.get("balances",[])
    template = templates_env.get_template("/admin/main.html")
    return template.render(
        {
         "request": request, 
         "page": "admin/create-task", 
         "title": Title, 
         "tag" : "Create Task",
         "data" :{"user": items["user"], 
                  "token": tokens
                  },  
         "user": items["user"],
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )

@adminview.get("/update-task/{task_id}", response_class=HTMLResponse)
async def update_task(task_id, request: Request, db: Session = Depends(get_db)):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/admin/login")
    """Password Reset View"""
    # print(items)
    task = await get_task_by_id(db=db, task_id=task_id, 
                           current_user = Security(user_service.get_current_user)
                           )
    template = templates_env.get_template("/admin/main.html")
    return template.render(
        {
         "request": request, 
         "page": "admin/update-task", 
         "title": Title, 
         "tag" : "Create Task",
         "data" :{"user": items["user"], "task":task},  
         "user": items["user"],
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )

@adminview.get("/submissions/{task_id}", response_class=HTMLResponse)
async def view_jobs(
    task_id, 
    request: Request, 
    db: Session = Depends(get_db),
):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/admin/login")
    
    template = templates_env.get_template("admin/main.html")
    #users = await get_users(db=db,page=1,per_page=10,is_active=None, is_deleted=None,is_verified=None, is_super_admin=None)

    tasks = await fetch_task_submissions(db=db, task_id=task_id,
    #current_user = Security(user_service.get_current_user)
    )
    if not tasks:
        tasks = []
    return template.render(
        {
         "request": request, 
         "page": "admin/submissions", 
         "title": Title, 
         "tag" : "All Tasks",
         "data" :{"tasks":tasks.get('data'),},  
         "user": items["user"],
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )


#Create Task View Admin
@adminview.get("/list-token", response_class=HTMLResponse)
def create_token(request: Request, db: Session = Depends(get_db)):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/admin/login")
    """Password Reset View"""
    # print(items)
    template = templates_env.get_template("/admin/main.html")
    return template.render(
        {
         "request": request, 
         "page": "admin/list-token", 
         "title": Title, 
         "tag" : "Create Task",
         "data" :{"user": items["user"]},  
         "user": items["user"],
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )

@adminview.get("/all/transactions", response_class=HTMLResponse)
def all_transaction(request: Request, db: Session = Depends(get_db)):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/login")
    """Transaction View"""
    template = templates_env.get_template("/admin/main.html")
    transactions = token_services.all_transactions(db)
    #print(transactions)
    return template.render(
        {
         "request": request, 
         "page": "admin/transactions", 
         "title": Title, 
         "tag" : "All transactions",
         "data" :{"user": items["user"], "transactions": transactions},  
         "user": items["user"],
         
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )
